﻿local VERSION = tonumber(("$Revision: 40605 $"):match("%d+"))

local QuestsFu = QuestsFu
local QuestsFu_Comments = QuestsFu:NewModule("Comments", "AceHook-2.1", "AceEvent-2.0")
if QuestsFu.revision < VERSION then
	QuestsFu.version = "r" .. VERSION
	QuestsFu.revision = VERSION
	QuestsFu.date = ("$Date: 2007-06-18 18:58:36 -0400 (Mon, 18 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local tablet = AceLibrary("Tablet-2.0")
local quixote = AceLibrary("Quixote-1.0")
local dewdrop = AceLibrary("Dewdrop-2.0")

local L = AceLibrary("AceLocale-2.2"):new("QuestsFu_Comments")
L:RegisterTranslations("enUS", function() return {
	["Comments"] = true,
	["Display quest comments."] = true,
	["No comments found..."] = true,
	["LightHeaded"] = true
} end)

L:RegisterTranslations("deDE", function() return {
	["Comments"] = "Kommentare",
	["Display quest comments."] = "Anzeigen von Quest Kommentaren.",
} end)

L:RegisterTranslations("koKR", function() return {
	["Comments"] = "설명",
	["Display quest comments."] = "퀘스트 설정을 표시합니다.",
} end)

L:RegisterTranslations("esES", function() return {
	["Comments"] = "Comentarios",
	["Display quest comments."] = "Muestra los comentarios de las misiones",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Comments"] = "備註",
	["Display quest comments."] = "顯示任務備註。",
	["No comments found..."] = "沒有備註...",
	["LightHeaded"] = "LightHeaded 資料庫"
} end)

function QuestsFu_Comments:OnInitialize()
	self.db = QuestsFu:AcquireDBNamespace("Comments")
	QuestsFu:RegisterDefaults("Comments", "profile", {
		lightheaded = true,
		data = {},
		strata = "HIGH",
	})
	QuestsFu.options.args.Comments = {
		name = L["Comments"],
		desc = L["Display quest comments."],
		type = "group",
		args = {
			lightheaded = {
				name = L["LightHeaded"],
				desc = L["LightHeaded"],
				type = 'toggle',
				get = function() return self.db.profile.lightheaded end,
				set = function(t) self.db.profile.lightheaded = t end,
				disabled = function() return not (LightHeaded and LightHeaded.IterateQuestComments) end
			},
			strata = {
				name=AceLibrary("AceLocale-2.2"):new("QuestsFu")["Strata"], type="text", desc=AceLibrary("AceLocale-2.2"):new("QuestsFu")["Adjust the strata of this panel (takes effect after you reload your UI)"],
				get=function() return self.db.profile.strata end,
				set=function(t) self.db.profile.strata = t end,
				validate={'BACKGROUND', 'LOW', 'MEDIUM', 'HIGH', },
				order = 1,
			},
			--This one will look a little weird on the tablet right-click menu, as it'll be duplicating native functionality.  But it's totally worth it to stop people complaining.
			lock = {
				type = 'toggle', name = AceLibrary("AceLocale-2.2"):new("QuestsFu")["Lock"],
				desc = AceLibrary("AceLocale-2.2"):new("QuestsFu")["Lock or unlock this panel"],
				disabled = function() return not tablet:IsRegistered("QuestsFu_Tracker") end,
				get = function() return tablet:IsRegistered("QuestsFu_Tracker") and tablet:IsLocked("QuestsFu_Tracker") end,
				set = function() tablet:ToggleLocked("QuestsFu_Tracker") end,
				order = 1,
			},
			toggle = {
				name = AceLibrary("AceLocale-2.2"):new("QuestsFu")["Enabled"],
				desc = AceLibrary("AceLocale-2.2"):new("QuestsFu")["Suspend/resume this module."],
				type = 'toggle',
				order = -1,
				get = function() return QuestsFu:IsModuleActive(self) end,
				set = function(t)
					if tablet:IsRegistered('QuestsFu_Comments') then
						if t then
							tablet:Open('QuestsFu_Comments')
						else
							tablet:Close('QuestsFu_Comments')
						end
					end
					QuestsFu:ToggleModuleActive(self)
				end
			},
		},
	}
end

function QuestsFu_Comments:OnEnable()
	if QuestsFu:IsModuleActive('Details') then
		self:Hook(QuestsFu, "ShowLog")
		self:Hook(QuestsFu:GetModule('Details'), "UpdateQuestDetail")
	end
end

function QuestsFu_Comments:ShowLog(object, questid)
	--self.questid = questid
	self.hooks[object].ShowLog(object, questid)
	if questid and LightHeaded and LightHeaded.IterateQuestComments and self.db.profile.lightheaded and QuestsFu:IsModuleActive('Details') then
		self:ScheduleEvent(self.Update, 0, self) --Schedule an event because tablet bugs out otherwise.
	end
end

function QuestsFu_Comments:UpdateQuestDetail(object)
	self.hooks[object].UpdateQuestDetail(object)
	self:Update()
end

function QuestsFu_Comments:Update()
	if not tablet:IsRegistered('QuestsFu_Comments') then
		local detail = QuestsFu:GetModule('Details')
		local function urlClick(url)
			ChatFrameEditBox:Show()
			ChatFrameEditBox:SetText(url)
			ChatFrameEditBox:SetFocus()
		end
		local function npcClick(npcid)
			if self.npcid == npcid then
				self.npcid = nil
			else
				self.npcid = npcid
			end
			self:Update()
		end
		local function addNpcLocs(cat, id, name)
			if LightHeaded:LoadNPCData(id) then
				for c,z,x,y in LightHeaded:IterateNPCLocs(id) do
					cat:AddLine('text', '|cFF0066FF['..x..', '..y..']|r', 'func', LightHeaded.OnHyperlinkClick, 'arg1', LightHeaded, 'arg2', 'lhref:zcoord:'..c..':'..z..':'..x..':'..y..':'..name, 'indentation', 10)
				end
			else
				cat:AddLine('text', 'Location unknown', 'indentation', 10)
			end
		end
		tablet:Register('QuestsFu_Comments', 'detachedData', self.db.profile.data, 'cantAttach', true, 'dontHook', true,
			'hideWhenEmpty', true, 'showTitleWhenDetached', true, 'strata', self.db.profile.strata, 'minWidth', 300,
			'menu', function() dewdrop:FeedAceOptionsTable(QuestsFu.options.args.Comments) end,
			'children', function()
				-- Much of this was taken from LightHeaded itself.
				if detail.questid and LightHeaded and LightHeaded.IterateQuestComments then
					local title, level, tag = quixote:GetQuestById(detail.questid)
					local qid,sharable,_,reqlev,stype,sname,sid,etype,ename,eid,exp,rep = LightHeaded:GetQuestInfo(title, level)
					
					tablet:SetTitle(L["Comments"])

					local cat = tablet:AddCategory('text', 'LightHeaded Info', 'columns', 1, 'wrap', true, 'justify', 'CENTER')
					
					cat:AddLine('text', 'Quest ID: |cffffffff'..qid, 'func', urlClick, 'arg1', 'http://www.wowhead.com/?quest='..qid)
					cat:AddLine('text', 'Shareable: |cffffffff'..(shareable and 'Yes' or 'No'))
					cat:AddLine('text', 'Level: |cffffffff'..level)
					if reqlev ~= '' then cat:AddLine('text', 'Required Level: |cffffffff'..reqlev) end
					if stype ~= '' and sname ~= '' then
						if stype=='npc' then
							cat:AddLine('text', 'Starts: |cFF0066FF['..sname..']|r', 'func', npcClick, 'arg1', sid) end
						else
							cat:AddLine('text', 'Starts: |cffffffff'..sname)
						end
					if sid == self.npcid then addNpcLocs(cat, sid, sname) end
					if etype ~= '' and ename ~= '' then
						if etype == 'npc' then
							cat:AddLine('text', 'Ends: |cFF0066FF['..ename..']|r', 'func', npcClick, 'arg1', eid) end
						else
							cat:AddLine('text', 'Ends: |cffffffff'..ename)
						end
					if eid == self.npcid then addNpcLocs(cat, eid, ename) end
					if exp ~= '' then cat:AddLine('text', 'Experience: |cffffffff'..exp) end
					if rep ~= '' then
						-- Might want to do an inline category here with columns...
						cat:AddLine('text', 'Reputation:')
						for name,value in rep:gmatch("([^\031]+)\031([^\031]+)") do
							cat:AddLine('text', '|cffffffff' .. name .. ' - ' .. value, 'indentation', 10)
						end
					end
					
					cat = tablet:AddCategory('text', 'LightHeaded Comments', 'columns', 1, 'wrap', true, 'justify', 'CENTER')
					if LightHeaded:GetNumQuestComments(title, level) then
						for idx, qid, cid, rating, indent, parent, date, poster, comment in LightHeaded:IterateQuestComments(title, level) do
							cat:AddLine('indentation', indent*10, 'text', ("|cFF66FF66[%s - %s - %s]|r"):format(poster, date, rating), 'wrap', true)
							cat:AddLine('indentation', indent*10, 'text', comment, 'wrap', true)
							cat:AddLine('text', ' ') -- spacer.
						end
					else
						cat:AddLine('text', L["No comments found..."])
					end
					
					cat:AddLine('text', "Comments are all from WowHead", 'hasCheck', true, 'checked', true, 'checkIcon', 'Interface\\Addons\\LightHeaded\\images\\wh_icon', 'func', urlClick, 'arg1', 'http://www.wowhead.com')
				end
			end)
		tablet:Open('QuestsFu_Comments')
	end
	tablet:Refresh('QuestsFu_Comments')
end
